#include <stdio.h>
#include <stdlib.h>
#include "liste.h"

void greska(){
	printf("-1\n");
	exit(EXIT_FAILURE);
}

cvor* novi_cvor(int vrednost){
	cvor *rez = malloc(sizeof(cvor));
	if(rez == NULL)
		greska();

	rez->vrednost = vrednost;
	rez->sledeci = NULL;

	return rez;
}

cvor* dodaj_na_pocetak(cvor *glava, int br){
	//mi imamo listu "lista"
	//a->b->c->d
	//i treba da dodajmo br, na pocetak liste
	//br->a->b->c->d
	cvor *novi = novi_cvor(br);
	novi->sledeci = glava;
	//nova glava liste je cvor "novi" koji samo sada kreirali
	//vracamo novu glavu liste
	return novi;
}

void dodaj_na_pocetak_2(cvor **glava, int br){
	//sada kada smo promenili sta saljemo cvor**, 
	//cvor* je glava liste, a cvor** adresa glave
	//samim tim glavu mozemo da izmenimo

	cvor* novi = novi_cvor(br);
	novi->sledeci = *glava;
	//menjamo glavu liste
	*glava = novi;
}

//ispis_liste_2 - iterativna varijanta - za vezbu kod kuce

void ispis_liste(cvor *glava){
	if(glava == NULL)
		return;
	//ispisujemo glavu, i pozivamo rekurzivno ispis ostatka liste
	printf("%d ", glava->vrednost);
	ispis_liste(glava->sledeci);	
}

//veoma bitna funkcija
//mi smo gore alocirali memoriju za nove cvorove, a nismo je 
//nikad oslobodili
//ni ne treba odmah da oslobadjamo memoriju, jer svi ti cvorovi ulaze u samu listu
//kada nam lista vise ne treba, onda pozivamo oslobadjanje svih cvorova


//lepa je praksa da vraca cvor*, iako to realno nije potrebno
//taj cvor* ce biti vrednost liste nakon oslobadjanja
//i uvek ce biti NULL
cvor* oslobodi(cvor *glava){
	if(glava == NULL)
		return NULL;
	//prvo oslobodimo ostatak liste, pa glavu.
	oslobodi(glava->sledeci);
	free(glava);	
	return NULL;
}

//oslobodi_2 - iterativna varijanta - za vezbu

//iterativna varijanta
cvor* dodaj_na_kraj2(cvor *lista, int br){
	//2 scenrija: 1) ako je lista prazna, 2) ako lista nije prazna
	if(lista == NULL)
		return novi_cvor(br);
	
	//ako lista nije prazna, moramo da dodjemo do poslednjeg elementa: tmp
	cvor *tmp = lista;
	while(tmp->sledeci != NULL)
		tmp = tmp->sledeci;

	//kada izadjemo iz ovog ciklusa, tmp je poslednji elemejnt liste
	tmp->sledeci = novi_cvor(br);
	return lista;
}

cvor* dodaj_na_kraj(cvor *lista, int br){
	if(lista == NULL)
		return novi_cvor(br);
	lista->sledeci = dodaj_na_kraj(lista->sledeci, br);
	return lista;
}

//dodaj_na_kraj3 - za vezbu kod kuce

cvor* napravi_listu(FILE *f){
	//ideja je ucitati listu iz datoteke
	cvor *lista = NULL; //MORAMO NA POCETKU DA JE INICIJALIUJEMO NA NULL
	int x;
	while(fscanf(f, "%d", &x) == 1)
		lista = dodaj_na_kraj(lista, x);
	return lista;
}

cvor* napravi_obrnutu_listu(FILE *f){
	cvor *lista = NULL;
	int x;
	while(fscanf(f, "%d", &x) == 1)
		lista = dodaj_na_pocetak(lista, x);
	return lista;
}

cvor* ucitaj_listu(char *file){
	FILE *f = fopen(file, "r");
	if(f == NULL)
		greska();
	cvor *lista = napravi_listu(f);
	fclose(f);
	return lista;
}

cvor* ucitaj_obrnutu_listu(char *file){
	FILE *f = fopen(file, "r");
	if(f == NULL)
		greska();
	cvor *lista = napravi_obrnutu_listu(f);
	fclose(f);
	return lista;
}

cvor* dodaj_sortirano(cvor *lista, int br){
	//podrazumeva se da je lista sortirana do sada
	//samo br treba da dodamo na ono mesto tako da lista ostane sortirana
	if(lista == NULL)
		return novi_cvor(br);
	//sada znamo da postoji lista, ali ne znamo da li postoji lista->sledeci
	if(br > lista->vrednost)
		lista->sledeci = dodaj_sortirano(lista->sledeci, br);	
	else
		lista = dodaj_na_pocetak(lista, br);
	return lista;
}

cvor* napravi_sortiranu_listu(FILE *f){
	cvor *lista = NULL;
	int x;
	while(fscanf(f, "%d", &x) == 1)
		lista = dodaj_sortirano(lista, x);
	return lista;	
}

cvor* ucitaj_sortiranu_listu(char *file){
	FILE *f = fopen(file, "r");
	if(f == NULL)
		greska();
	cvor *lista = napravi_sortiranu_listu(f);
	fclose(f);
	return lista;	
}

cvor* spoj_sortirano(cvor *l1, cvor *l2){
	if(l1 == NULL)
		return l2;
	if(l2 == NULL)
		return l1;

	//nakon ova dva if-a ZNAMO da ni l1 ni l2 nisu NULL
	//i tek onda mozemo da pisemo l1->vrenost, l2->vrednost
	if(l1->vrednost < l2->vrednost){
		l1->sledeci = spoj_sortirano(l1->sledeci, l2);
		return l1;
	}
	else {
		l2->sledeci = spoj_sortirano(l1, l2->sledeci);
		return l2;
	}		
}


cvor* izbaci_el(cvor *l, int br){
	//izbacujemo prvo pojavljivanje elementa koji ima vrednost br
	//iz liste
	//za vezbu probati iterativno
	//takodje za vezbu probati izbacivanje svako pojavljivanje elementa br
	if(l == NULL)
		return NULL;

	if(l->vrednost == br){
		cvor *tmp = l->sledeci;
		free(l);
		return tmp;
	}
	else{
		l->sledeci = izbaci_el(l->sledeci, br);
		return l;
	}
}
